# -*- coding: utf-8 -*-
'''
HTML Object Printer, HOP in short!
Helper for rendering safe html objects with ease.
'''

__author__ = u'Michał Thoma'
__author_website__ = u'http://balor.pl'
__version__ = u'1.5.2'
__website__ = u'https://github.com/balor/hop'



# -*- coding: utf-8 -*-

import tags

#TODO: write this helper...

class Helper(object):

    def __init__(self, website_name, website_url):
        self.website_name = website_name
        self.website_url = website_url



# -*- coding: utf-8 -*-

from ..builder import (
    build_html_object,
    build_html_object_close_tag,
    build_html_object_open_tag,
    build_html_self_closing_object,
)
from ..utils import (
    mk_literal,
    safestr, 
    sstrip,
    validate_url,
)


def a(href, body=None, title=None, **params):
    if 'validate_url' in params:
        params.pop('validate_url')
        href = validate_url(href, special_protocols=True)
    else:
        href = sstrip(href)

    if not body:
        body = safestr(href)
    if title == None:
        title = safestr(body)

    params['href'] = href
    params['title'] = title

    if title == False:
        params.pop('title')

    return build_html_object(u'a', body, **params)


def img(src, alt=None, **params):
    if 'validate_url' in params:
        params.pop('validate_url')
        params['src'] = validate_url(src, special_protocols=True)
    else:
        params['src'] = sstrip(src)

    if alt:
        params['alt'] = alt
    else:
        params['alt'] = u''
    return build_html_self_closing_object(u'img', **params)


def style(href, **params):
    if 'validate_url' in params:
        params.pop('validate_url')
        href = validate_url(href, special_protocols=True)
    else:
        href = sstrip(href)

    params.update({
        'href': href,
        'rel': u'stylesheet',
        'type': u'text/css',
    })
    return build_html_self_closing_object(u'link', **params)


def script(src, **params):
    if 'validate_url' in params:
        params.pop('validate_url')
        src = validate_url(src, special_protocols=True)
    else:
        src = sstrip(src)

    params.update({
        'src': src,
        'type': u'text/javascript',
    })
    return build_html_object(u'script', u'', **params)


def meta_charset(charset='utf-8'):
    params = {
        'content': u'text/html; charset={0}'.format(charset),
        'http-equiv': u'Content-Type',
    }
    return build_html_self_closing_object(u'meta', **params)


def title(title):
    return build_html_object('title', title)


def comment(comment):
    return mk_literal(u'<!-- {0} -->'.format(comment))


def button(body=u'', **params):
    if not 'type' in params:
        params['type'] = u'button'
    return build_html_object(u'button', body, **params)


def html_list(items=list(), list_type='ul', **params):
    '''
    Item param can be string or list
        with tuples/lists looking like this:

        [(item1_body, item1_options),(item2_body, item2_options)]
    '''
    out_html = [
        build_html_object_open_tag(list_type, **params),
    ]
    for item in items:
        body = u'&nbsp;'
        params = dict()

        if type(item) == list or type(item) == tuple:
            item_len = len(item)
            if item_len > 0:
                body = item[0]
            if item_len > 1:
                params = item[1]
        elif sstrip(item) != u'':
            body = sstrip(item)

        li_obj = build_html_object(u'li', body, **params)
        out_html.append(li_obj)

    closing_tag = build_html_object_close_tag(list_type)
    out_html.append(closing_tag)
    return mk_literal(u''.join(out_html))


##########################
# aliases and wrappers

link = a
charset = meta_charset

def email(email, body=None, title=None, **params):
    href = u'mailto:{0}'.format(email)
    if not body:
        body = email
    if not title:
        title = u'Mail to {0}'.format(email)
    return a(href, body, title, **params)

def ul(items=list(), **params):
    return html_list(items, u'ul', **params)

def ol(items=list(), **params):
    return html_list(items, u'ol', **params)



# -*- coding: utf-8 -*-

from utils import (
    escape, 
    mk_literal,
    sstrip,
)


def build_html_params(**params):
    params_list = list()
    for (name, value) in params.items():
        if name[0] == u'_':
            name = name[1:]
        params_list.append(u' {0}="{1}"'.format(
            escape(name), 
            escape(value),
        ))
    return u''.join(params_list)


def build_html_object_open_tag(object_name, **params):
    return mk_literal(u'<{0}{1}>'.format(
        sstrip(escape(object_name)),
        build_html_params(**params),
    ))


def build_html_object_close_tag(object_name):
    return mk_literal(u'</{0}>'.format(sstrip(escape(object_name))))


def build_html_object(object_name, body, **params):
    return mk_literal(u''.join([
        build_html_object_open_tag(object_name, **params),
        escape(body),
        build_html_object_close_tag(object_name),
    ]))


def build_html_self_closing_object(object_name, **params):
    return mk_literal(u'<{0}{1} />'.format(
        sstrip(escape(object_name)),
        build_html_params(**params),
    ))



# -*- coding: utf-8 -*-

from constants import (
    EMPTY,
    SPECIAL_URL_PROTOCOLS,
    URL_PROTOCOLS,
)

try:
    from markupsafe import escape_silent as escape
except ImportError:
    try:
        import markupsafe
        def escape(string):
            if string is None:
                return EMPTY
            return markupsafe.escape(string)
    except ImportError:
        def escape(string):
            if string is None:
                return EMPTY
            if hasattr(string, '__html__'):
                return string
            return (string.
                replace(u'\n', u'').
                replace(u'\r', u'').
                replace(u'<', u'&lt;').
                replace(u'>', u'&gt;').
                replace(u'"', u'&#34;'))


class Literal(unicode):
    '''
    Literal class represents the unicode string
    not intented for html-escapeing.

    This object is designed to fit to the auto-escaping
    mechanisms used in pylons, pyramid, mako etc...
    '''
    tag = u''

    def __init__(self, tag):
        self.tag = tag

    def __html__(self):
        return self.tag

    def __str__(self):
        return self.tag


def mk_literal(html):
    '''
    Returns a new instance of Literal class based on given html string.

    Params:
        html - unicode string
    '''
    return Literal(html)


def sstrip(obj):
    '''
    Small function converting any non-string objects to
    string, removing all double-quotes from it and
    stripping it from whitespaces from beginning and end.

    Params:
        obj - object to be stringyfied and stripper
    '''
    if not isinstance(obj, basestring):
        obj = str(obj)
    return obj.strip().strip('"')


def safestr(string):
    '''
    Escapes all xml non-safe characters in string.

    Params:
        string - string that will be escaped
    '''
    return (string.
        replace(u'&', u'&amp;').
        replace(u'"', u'&quot;').
        replace(u'<', u'&lt;').
        replace(u'>', u'&gt;'))


def validate_url(url, special_protocols=False, base_path=None):
    '''
    Safe strips the url and validates its protocol.
    
    Examples:
        >> validate_url(u'http://www.example.com')
        u'http://www.example.com'

        >> validate_url(u'\thttp://www.example.com   ')
        u'http://www.example.com'

        >> validate_url(u'example.com  ')
        u'http://example.com'

        >> validate_url(u'/users/sign_in', base_path='http://example.com')
        u'http://example.com/users/sign_in'

        >> validate_url(u'gg:123123123', True, 'http://example.com')
        u'gg:123123123'

    Params:
        url - string containing a url
        special_protocols - use special protocols (False by default)
        base_path - path used on auto-completing the url
    '''
    url = sstrip(url)
    valid_url_protocols = URL_PROTOCOLS
    if special_protocols:
        valid_url_protocols.extend(SPECIAL_URL_PROTOCOLS)

    url_has_valid_protocol = False
    for protocol in valid_url_protocols:
        if url.startswith(protocol):
            url_has_valid_protocol = True

    if url_has_valid_protocol:
        return url

    url = url[1:] if url and url[0] == '/' else url
    if base_path:
        base_path = base_path[:-1] if base_path[-1] == '/'  else base_path
        url = u'{0}{1}'.format(base_path, url)
    else:
        url = u'http://{0}'.format(url)

    return url



# -*- coding: utf-8 -*-

from ..builder import (
    build_html_object,
    build_html_object_close_tag,
    build_html_object_open_tag,
)
from ..utils import (
    mk_literal,
    sstrip,
)


def _cells_iterator(row, columns_num=None):
    if not columns_num:
        for cell in row:
            yield cell
    else:
        row_len = len(row)
        for x in range(columns_num):
            if x < row_len:
                yield row[x]
            else:
                yield mk_literal(u'&nbsp;')


def table(cells, headers=None, **params):
    headers_are_invalid = bool(headers and not isinstance(headers, list))
    if not isinstance(cells, list) or headers_are_invalid:
        raise TypeError('Cells and headers must be a valid list object')

    out_html = list()

    fixed_columns_num = False
    if 'fixed_columns_num' in params:
        fixed_columns_num = int(params.pop('fixed_columns_num'))

    out_html.append(build_html_object_open_tag(u'table', **params))

    if headers:
        out_html.append(build_html_object_open_tag(u'tr'))

        for header in _cells_iterator(headers, fixed_columns_num):
            out_html.append(build_html_object(u'th', header))

        out_html.append(build_html_object_close_tag(u'tr'))

    for row in cells:
        tr_params = dict()
        tds = list()

        for cell in _cells_iterator(row, fixed_columns_num):
            if isinstance(cell, dict):
                if 'tr' in cell:
                    tr_params = cell['tr']
            else:
                td_params = dict()
                if isinstance(cell, tuple):
                    cell = cell[0]
                    td_params = cell[1]
                td_obj = build_html_object(u'td', cell, **td_params)
                tds.append(td_obj)

        tr_opening = build_html_object_open_tag(u'tr', **tr_params)
        tr_closing = build_html_object_close_tag(u'tr')
        row_html = u''.join(tds)
        out_html.append(u'{0}{1}{2}'.format(tr_opening, row_html, tr_closing))

    out_html.append(build_html_object_close_tag(u'table'))

    return mk_literal(u''.join(out_html))


def fixed_col_table(cells, columns_num, headers=None, **params):
    params['fixed_columns_num'] = columns_num
    return table(cells, headers, **params)



# -*- coding: utf-8 -*-

from ..builder import (
    build_html_object,
    build_html_object_close_tag,
    build_html_object_open_tag,
    build_html_self_closing_object,
)
from ..utils import (
    mk_literal,
    safestr, 
    sstrip,
)


def label(body, _for=None, **params):
    if _for:
        params['for'] = _for
    return build_html_object(u'label', body, **params)


def _set_id_and_name(name, params):
    if name:
        params['name'] = name
        if not 'id' in params:
            params['id'] = name
    elif 'id' in params:
        params['name'] = params['id']
    return params


def _set_label(params):
    label_obj = u''
    if 'label' in params:
        label_body = params['label']
        params.pop('label')
        if isinstance(label_body, bool) and label_body:
            label_obj = label(params.get('id'), params['name'])
        else:
            label_obj = label(label_body, params['name'])
    return label_obj, params


def form(action=u'#', method='post', multipart=False, **params):
    if not params.has_key('accept-charset'):
        params['accept-charset'] = u'utf-8'
    if multipart:
        params['enctype'] = u'multipart/form-data'

    params['action'] = action
    params['method'] = method

    return build_html_object_open_tag('form', **params)


def end_form():
    return build_html_object_close_tag('form')


def textarea(name, body=u'', **params):
    params = _set_id_and_name(name, params)
    label_obj, params = _set_label(params)

    txt_area_obj = build_html_object(u'textarea', body, **params)
    return mk_literal(u'{0}{1}'.format(label_obj, txt_area_obj))


def select(name, items=None, selected=None, **params):

    html = list()

    if not items:
        items = list()

    if not isinstance(items, list):
        raise TypeError('items must be an instance of list')

    params = _set_id_and_name(name, params)
    label_obj, params = _set_label(params)

    select_open_tag = build_html_object_open_tag('select', **params) 
    html.append(select_open_tag)

    if items and not isinstance(items[0], tuple):
        temp_items = list()
        for item in items:
            temp_items.append((item, item))
        items = temp_items

    if not selected and items:
        selected = items[0][0]
    selected = sstrip(selected)

    for value, body in items:
        item_params = dict(value=sstrip(value))

        if item_params['value'] == selected:
            item_params['selected'] = u'selected'

        option_obj = build_html_object(u'option', body, **item_params)
        html.append(option_obj)

    html.append(build_html_object_close_tag('select'))
    select_obj = u''.join(html)
    return mk_literal(u'{0}{1}'.format(label_obj, select_obj))


def input_field(type, name=None, value=None, **params):
    params['type'] = type
    params['value'] = value if value else u''

    params = _set_id_and_name(name, params)
    label_obj, params = _set_label(params)

    if type in ['radio', 'checkbox'] and 'checked' in params:
        if params.pop('checked'):
            params['checked'] = u'checked'

    input_obj = build_html_self_closing_object(u'input', **params)
    if type == 'hidden':
        input_obj = build_html_object(
            u'div', input_obj, style=u'display:none;')

    return mk_literal(u'{0}{1}'.format(label_obj, input_obj))


def text(name, value=u'', **params):
    return input_field(
        type = u'text',
        name = name,
        value = sstrip(value),
        **params
    )


def password(name, value=u'', **params):
    return input_field(
        type = u'password',
        name = name,
        value = sstrip(value),
        **params
    )


def file(name, **params):
    return input_field(
        type = u'file',
        name = name,
        **params
    )


def hidden(name, value=u'', **params):
    return input_field(
        type = u'hidden',
        name = name,
        value = sstrip(value),
        **params
    )


def checkbox(name, value=u'true', checked=False, **params):
    if checked:
        params['checked'] = True

    return input_field(
        type = u'checkbox',
        name = name,
        value = sstrip(value),
        **params
    )


def radio(name, value=u'', checked=False, **params):
    if checked:
        params['checked'] = True

    return input_field(
        type = u'radio',
        name = name,
        value = value,
        **params
    )


def radio_list(name, values, value_checked=None, **params):
    '''
    Prints the list of radio buttons

    name - name for the buttons
    values - can be list or dict:
        
        ['val_0', 'val_1', 'val_2']

        {
            'val_0': u'Label zero',
            'val_1': u'Label one',
            'val_2': u'Label two',
        }
    '''
    html = list()
    def make_list_item(val, label_str=None):
        label_str = label_str if label_str else val
        field = u'{0}{1}'.format(
            radio(name, val, val==value_checked, **params), label_str) 
        return label(mk_literal(field))

    for value in values:
        if isinstance(value, list):
            value, label_str = value[0], value[1]
        else:
            label_str = value
        html.append(make_list_item(value, label_str))

    return mk_literal(u''.join(html))


def submit(name=None, value=None, **params):
    name = value if name == None else name
    value = name if value == None else value

    return input_field(
        type = u'submit',
        name = name,
        value = value,
        **params
    )



# -*- coding: utf-8 -*-

EMPTY = u''

URL_PROTOCOLS = [
    u'cvs://',
    u'e2k://',
    u'feed:',
    u'file://',
    u'ftp://',
    u'git://',
    u'go://',
    u'gopher://',
    u'http://',
    u'https://',
    u'imap://',
    u'irc://',
    u'ircs://',
    u'ldap://',
    u'nntp://',
    u'pop://',
    u'rmi://',
    u'rsync://',
    u'sftp://',
    u'ssh://',
    u'teamspeak://',
]

SPECIAL_URL_PROTOCOLS = [
    u'#',
    u'aim:',
    u'apt:',
    u'cid:',
    u'dav:',
    u'dns:',
    u'fax:',
    u'gg:',
    u'gtalk:',
    u'im:',
    u'mailto:',
    u'maps:',
    u'mid:',
    u'news:',
    u'skype:',
    u'tag:',
    u'tel:',
    u'telnet:',
    u'view-source:',
    u'ws:',
    u'xmpp:',
]




# -*- coding: utf-8 -*-
import sys
import os
from distutils.core import setup

if sys.version_info < (2,6):
    raise NotImplementedError(u'You need Python 2.6 or higher to use hop.')

from hop import __version__, __author__, __website__


setup(
    name = 'hop',
    version = str(__version__),
    description = (
        u'HTML Object Printer - simple web helpers without dependencies.'
    ),
    author = __author__,
    author_email = 'michal@balor.pl',
    license = 'MIT',
    packages = ['hop', 'hop.tags'],
    platforms = ['any'],
    url = str(__website__),
)
